==> mediamvp_0.0.3.tar.gz: mediamvp/menu.c <==
/*
 *   MediaMVP Server
 *
 *   (C) 2003 Dominic Morris
 *
 *   $Id: menu.c,v 1.13 2003/11/24 20:18:40 dom Exp $
 *   $Date: 2003/11/24 20:18:40 $
 *
 *
 *   Constructs a menu and all such things
 *
 */

#include "mediamvp.h"
#include "keys.h"


/* Number of rows we can display on the menu */

#define BLACK   0x00, 0x00, 0x00
#define RED     0xFC, 0x14, 0x14
#define GREEN   0x24, 0xFC, 0x24
#define YELLOW  0xFC, 0xC0, 0x24
#define BLUE    0x00, 0x00, 0xFC
#define CYAN    0x00, 0xFC, 0xFC
#define MAGENTA 0xB0, 0x00, 0xFC
#define WHITE   0xFC, 0xFC, 0xFC


typedef struct {
    char    *name;
    int    (*callback_func)(menu_t *menu,app_t *app, int sel);
    int      num_options;
    char   **options;
    int      type;
    void    *value_ptr;
} option_t;


struct _menu {
    render_t     *render;
    app_t        *app;
    gui_t        *gui;
    int           numoptions;
    option_t     *options;
    char         *title;
    int           top_offset;
    int           current_selection;
    int           maxrows;                /*!< Number of lines that can be displayed on screen */
    int           flags;
    menu_t        *parent;
    menu_t        *child;
};

static int        menu_keys(void *ptr, int code);



menu_t *new_menu(render_t *render, menu_t *parent, app_t *app, gui_t *gui, int flags)
{
    menu_t       *menu = malloc(sizeof(*menu));

    menu->render = render;
    menu->app    = app;
    menu->gui    = gui;
    menu->title  = NULL;
    menu->top_offset = 0;
    menu->numoptions = 0;
    menu->options = NULL;
    menu->current_selection = 0;
    menu->flags   = flags;
    menu->maxrows = render_get_maxrows(render);

    if ( parent ) {
        menu->parent = parent;
        parent->child = menu;
    } else {
        menu->parent = NULL;
    }
    menu->child = NULL;

    return menu;
}

void delete_menu_tree(menu_t *menu)
{
    menu_t   *temp;

    while ( menu->child ) {
	menu = menu->child;
    }

    while ( menu->parent ) {
	temp = menu->parent;
	delete_menu(temp);
	menu = temp;
    }

}

void delete_menu(menu_t *menu)
{
    int     i;

    if ( menu->title ) {
        free(menu->title);
        menu->title = NULL;
    }

    for ( i = 0; i < menu->numoptions; i++ ) {
        free(menu->options[i].name);
    }
    if ( menu->options ) {
        free(menu->options);
    }
}

void menu_set_title(menu_t *menu, char *text)
{
    if ( menu->title ) {
        free(menu->title);
    }

    menu->title = strdup(text);
}


void menu_add_option(menu_t *menu, char *name, int (callback_func)(menu_t *menu,app_t *app, int sel),
                     int type, int num_options, char **options, void *value_ptr)
{
    int     i = menu->numoptions++;

    menu->options = realloc(menu->options,menu->numoptions * sizeof(option_t));

    menu->options[i].name = strdup(name);
    menu->options[i].callback_func = callback_func;
    menu->options[i].type = type;
    menu->options[i].num_options = num_options;
    menu->options[i].options = options;
    menu->options[i].value_ptr = value_ptr;
}

void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t *app, int sel))
{
    int     i = menu->numoptions++;

    menu->options = realloc(menu->options,menu->numoptions * sizeof(option_t));

    menu->options[i].name = strdup(name);
    menu->options[i].callback_func = callback_func;
    menu->options[i].type          = MENU_NONE;
    menu->options[i].num_options = 0;
    menu->options[i].value_ptr = NULL;
}




void menu_display(menu_t *menu)
{
    char          buf[256];
    char         *extra;
    render_t     *render = menu->render;
    int           row;
    int           i;
    int           last;

    while ( menu->child ) {
        menu = menu->child;
    }

    printf("Displaying menu\n");
    render_set_bgcol(render,BLACK);
    render_clear(render);

    snprintf(buf,sizeof(buf)," %s\n",menu->title ? menu->title : "");
    render_set_bgcol(render,CYAN);
    render_set_fgcol(render,BLACK);
    render_printrow(render,0,buf);


    /* Calculate how many rows to print */
    last = (menu->top_offset + (menu->maxrows - 2) < menu->numoptions) ? menu->top_offset + (menu->maxrows - 2) : menu->numoptions;
    for ( row = 2, i = menu->top_offset; i < last; i++, row++ ) {
        if ( menu->options[i].type == MENU_INT ) {
	    printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
            extra = menu->options[i].options[*((int *)menu->options[i].value_ptr)];
        } else {
            extra = "";
        }
        if ( i < 9 && menu->flags & MENU_NUMBERS ) {
            snprintf(buf,sizeof(buf)," %d. %s %s\n",i+1,menu->options[i].name,extra);
        } else {
            snprintf(buf,sizeof(buf),"   %s %s\n",menu->options[i].name,extra);
        }      

        if ( i == menu->current_selection ) {
            render_set_bgcol(render,CYAN);
            render_set_fgcol(render,BLACK);
        } else {
            render_set_bgcol(render,BLACK);
            render_set_fgcol(render,WHITE);
        }
        render_printrow(render,row,buf);
    }
    gui_register_keys(menu->gui,REGISTER_MENU,menu_keys,menu);
}


static int menu_keys(void *ptr, int code)
{
    menu_t    *menu = ptr;
    int        pos;
    int      (*func)(menu_t *,app_t *app, int sel);
    option_t  *option;
    unsigned int       *num;


    printf("Got menu key %d\n",code);
    switch ( code ) {
    case kChanUp:
        if ( menu->current_selection > 0 ) {
            menu->current_selection--;
            if ( menu->top_offset > menu->current_selection ) {
                menu->top_offset = menu->current_selection;
            }
        } else {
            menu->current_selection = menu->numoptions - 1;	  
            if ( menu->numoptions > menu->maxrows - 2 ) {
                menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
            }
        }
        menu_display(menu);
        break;
    case kChanDn:
        if ( menu->current_selection < menu->numoptions - 1 ) {
            menu->current_selection++;
            if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
                menu->top_offset++;
            }
        } else {
            menu->current_selection = 0;
            menu->top_offset = 0;
        }
        menu_display(menu);
        break;
    case kVolDn:
        option = &menu->options[menu->current_selection];
        if ( option->type == MENU_INT ) {
	    menu_t *parent = menu->parent;
            num = option->value_ptr;           
            *num = ( *num + 1 ) % option->num_options;
            func = menu->options[menu->current_selection].callback_func;
            if ( func && func(menu,menu->app,-1) == 0 ) {
                delete_menu(menu);
                parent->child = NULL;	
            }    
	    menu_display(parent);
  
        }
        break;
    case kVolUp:
        option = &menu->options[menu->current_selection];
        if ( option->type == MENU_INT ) {
	    menu_t *parent = menu->parent;
            num = option->value_ptr;
            if ( *num == 0 ) {
                *num = option->num_options - 1;
            } else {
                *num = ( *num - 1 ) % option->num_options;
            }
            func = menu->options[menu->current_selection].callback_func;
            if ( func && func(menu,menu->app,-1) == 0 ) {
                delete_menu(menu);
                parent->child = NULL;	
            }      
	    menu_display(parent);

        }
        break;
    case kBack:
        if ( menu->parent ) {
            menu_t *parent = menu->parent;
            delete_menu(menu);
            parent->child = NULL;
            menu_display(parent);
        }
        break;
    case kGo:
        while ( menu->parent ) {
            menu_t *parent = menu->parent;
            delete_menu(menu);
            menu = parent;
            menu->child = NULL;
        }
        menu_display(menu);
        break;
    case kOk:
        if ( menu->options != NULL ) {
            menu_t  *parent;
            func = menu->options[menu->current_selection].callback_func;
            if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
                parent = menu->parent;
                delete_menu(menu);
                parent->child = NULL;	
                menu_display(parent);
            }        
        }
        break;
    case k1 ... k9:
        if ( menu->flags & MENU_NUMBERS ) {
            pos = code - k1;
            if ( pos < menu->numoptions && menu->options ) {
                func = menu->options[pos].callback_func;
                if ( func && func(menu,menu->app,pos) == 0 ) {
                    delete_menu(menu);
                }
            }
        }
        break;
    default:
        printf("Received IR code %d - not used here\n",code);
    }
    return 0;
}


 
/*
 * Local Variables:
 *  indent-tabs-mode:nil
 *  require-final-newline:t
 *  c-basic-offset: 4
 * End:
 */

==> mediamvp_0.0.4.tar.gz: mediamvp/menu.c <==
/*
 *   MediaMVP Server
 *
 *   (C) 2003 Dominic Morris
 *
 *   $Id: menu.c,v 1.14 2003/11/24 21:52:00 dom Exp $
 *   $Date: 2003/11/24 21:52:00 $
 *
 *
 *   Constructs a menu and all such things
 *
 */

#include "mediamvp.h"
#include "keys.h"


/* Number of rows we can display on the menu */

#define BLACK   0x00, 0x00, 0x00
#define RED     0xFC, 0x14, 0x14
#define GREEN   0x24, 0xFC, 0x24
#define YELLOW  0xFC, 0xC0, 0x24
#define BLUE    0x00, 0x00, 0xFC
#define CYAN    0x00, 0xFC, 0xFC
#define MAGENTA 0xB0, 0x00, 0xFC
#define WHITE   0xFC, 0xFC, 0xFC


typedef struct {
    char    *name;
    int    (*callback_func)(menu_t *menu,app_t *app, int sel);
    int      num_options;
    char   **options;
    int      type;
    void    *value_ptr;
} option_t;


struct _menu {
    dongle_t     *dongle;
    render_t     *render;
    app_t        *app;
    gui_t        *gui;
    int           numoptions;
    option_t     *options;
    char         *title;
    int           top_offset;
    int           current_selection;
    int           maxrows;                /*!< Number of lines that can be displayed on screen */
    int           flags;
    menu_t        *parent;
    menu_t        *child;
};

static int        menu_keys(void *ptr, int code);



menu_t *new_menu(dongle_t *dongle,render_t *render, menu_t *parent, app_t *app, int flags)
{
    menu_t       *menu = malloc(sizeof(*menu));

    menu->dongle = dongle;
    menu->render = render;
    menu->app    = app;
    menu->gui    = dongle->gui;
    menu->title  = NULL;
    menu->top_offset = 0;
    menu->numoptions = 0;
    menu->options = NULL;
    menu->current_selection = 0;
    menu->flags   = flags;
    menu->maxrows = render_get_maxrows(render);

    if ( parent ) {
        menu->parent = parent;
        parent->child = menu;
    } else {
        menu->parent = NULL;
    }
    menu->child = NULL;

    return menu;
}

void delete_menu_tree(menu_t *menu)
{
    menu_t   *temp;

    while ( menu->child ) {
	menu = menu->child;
    }

    while ( menu->parent ) {
	temp = menu->parent;
	delete_menu(temp);
	menu = temp;
    }

}

void delete_menu(menu_t *menu)
{
    int     i;

    if ( menu->title ) {
        free(menu->title);
        menu->title = NULL;
    }

    for ( i = 0; i < menu->numoptions; i++ ) {
        free(menu->options[i].name);
    }
    if ( menu->options ) {
        free(menu->options);
    }
}

void menu_set_title(menu_t *menu, char *text)
{
    if ( menu->title ) {
        free(menu->title);
    }

    menu->title = strdup(text);
}


void menu_add_option(menu_t *menu, char *name, int (callback_func)(menu_t *menu,app_t *app, int sel),
                     int type, int num_options, char **options, void *value_ptr)
{
    int     i = menu->numoptions++;

    menu->options = realloc(menu->options,menu->numoptions * sizeof(option_t));

    menu->options[i].name = strdup(name);
    menu->options[i].callback_func = callback_func;
    menu->options[i].type = type;
    menu->options[i].num_options = num_options;
    menu->options[i].options = options;
    menu->options[i].value_ptr = value_ptr;
}

void menu_add(menu_t *menu, char *name, int (*callback_func)(menu_t *menu,app_t *app, int sel))
{
    int     i = menu->numoptions++;

    menu->options = realloc(menu->options,menu->numoptions * sizeof(option_t));

    menu->options[i].name = strdup(name);
    menu->options[i].callback_func = callback_func;
    menu->options[i].type          = MENU_NONE;
    menu->options[i].num_options = 0;
    menu->options[i].value_ptr = NULL;
}




void menu_display(menu_t *menu)
{
    char          buf[256];
    char         *extra;
    render_t     *render = menu->render;
    int           row;
    int           i;
    int           last;

    while ( menu->child ) {
        menu = menu->child;
    }

    printf("Displaying menu\n");
    render_set_bgcol(render,BLACK);
    render_clear(render);

    snprintf(buf,sizeof(buf)," %s\n",menu->title ? menu->title : "");
    render_set_bgcol(render,CYAN);
    render_set_fgcol(render,BLACK);
    render_printrow(render,0,buf);


    /* Calculate how many rows to print */
    last = (menu->top_offset + (menu->maxrows - 2) < menu->numoptions) ? menu->top_offset + (menu->maxrows - 2) : menu->numoptions;
    for ( row = 2, i = menu->top_offset; i < last; i++, row++ ) {
        if ( menu->options[i].type == MENU_INT ) {
	    printf("option %d val %d\n",i,*((int *)menu->options[i].value_ptr));
            extra = menu->options[i].options[*((int *)menu->options[i].value_ptr)];
        } else {
            extra = "";
        }
        if ( i < 9 && menu->flags & MENU_NUMBERS ) {
            snprintf(buf,sizeof(buf)," %d. %s %s\n",i+1,menu->options[i].name,extra);
        } else {
            snprintf(buf,sizeof(buf),"   %s %s\n",menu->options[i].name,extra);
        }      

        if ( i == menu->current_selection ) {
            render_set_bgcol(render,CYAN);
            render_set_fgcol(render,BLACK);
        } else {
            render_set_bgcol(render,BLACK);
            render_set_fgcol(render,WHITE);
        }
        render_printrow(render,row,buf);
    }
    gui_register_keys(menu->gui,REGISTER_MENU,menu_keys,menu);
}


static int menu_keys(void *ptr, int code)
{
    menu_t    *menu = ptr;
    int        pos;
    int      (*func)(menu_t *,app_t *app, int sel);
    option_t  *option;
    unsigned int       *num;
    int      streamtype = 0;


    if ( menu->dongle->stream ) {
	streamtype = stream_get_type(menu->dongle->stream);
    }

    printf("Got menu key %d\n",code);

    /* If we're in streaming mpeg mode, then we'd like to change channels
       without going back to the menu 
    */
    if ( streamtype == ( MEDIA_MPEG | MEDIA_SOCKET ) ) {
	switch ( code ) {
	case kChanUp:
	    if ( menu->current_selection < menu->numoptions - 1 ) {
		menu->current_selection++;
		if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
		    menu->top_offset++;
		}
	    } else {
		menu->current_selection = 0;
		menu->top_offset = 0;
	    }

	    func = menu->options[menu->current_selection].callback_func;
	    if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
		menu_t  *parent;		
		parent = menu->parent;
		delete_menu(menu);
		parent->child = NULL;	
	    }  
	    break;
	case kChanDn:
	    if ( menu->current_selection > 0 ) {
		menu->current_selection--;
		if ( menu->top_offset > menu->current_selection ) {
		    menu->top_offset = menu->current_selection;
		}
	    } else {
		menu->current_selection = menu->numoptions - 1;	  
		if ( menu->numoptions > menu->maxrows - 2 ) {
		    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
		}
	    }
	    func = menu->options[menu->current_selection].callback_func;
	    if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
		menu_t  *parent;		
		parent = menu->parent;
		delete_menu(menu);
		parent->child = NULL;	
	    }  
	    break;
	}
	return 0;
    }



    switch ( code ) {
    case kChanUp:
	if ( streamtype != MEDIA_MPEG ) {
	    if ( menu->current_selection > 0 ) {
		menu->current_selection--;
		if ( menu->top_offset > menu->current_selection ) {
		    menu->top_offset = menu->current_selection;
		}
	    } else {
		menu->current_selection = menu->numoptions - 1;	  
		if ( menu->numoptions > menu->maxrows - 2 ) {
		    menu->top_offset = menu->numoptions - ( menu->maxrows - 2);
		}
	    }
	    
	    menu_display(menu);
	}
        break;
    case kChanDn:
	if ( streamtype != MEDIA_MPEG ) {
	    if ( menu->current_selection < menu->numoptions - 1 ) {
		menu->current_selection++;
		if ( menu->top_offset + (menu->maxrows - 2) <= menu->current_selection ) {
		    menu->top_offset++;
		}
	    } else {
		menu->current_selection = 0;
		menu->top_offset = 0;
	    }

	    if ( streamtype == ( MEDIA_MPEG | MEDIA_SOCKET ) ) {
		menu_t  *parent;
		func = menu->options[menu->current_selection].callback_func;
		if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
		    parent = menu->parent;
		    delete_menu(menu);
		    parent->child = NULL;	
		    menu_display(parent);
		}  
	    } else if ( (streamtype & MEDIA_MASK) != MEDIA_MPEG ) {
		menu_display(menu);
	    }
	}
        break;
    case kVolDn:
        option = &menu->options[menu->current_selection];
        if ( option->type == MENU_INT ) {
	    menu_t *parent = menu->parent;
            num = option->value_ptr;           
            *num = ( *num + 1 ) % option->num_options;
            func = menu->options[menu->current_selection].callback_func;
            if ( func && func(menu,menu->app,-1) == 0 ) {
                delete_menu(menu);
                parent->child = NULL;	
            }    
	    menu_display(parent);
  
        }
        break;
    case kVolUp:
        option = &menu->options[menu->current_selection];
        if ( option->type == MENU_INT ) {
	    menu_t *parent = menu->parent;
            num = option->value_ptr;
            if ( *num == 0 ) {
                *num = option->num_options - 1;
            } else {
                *num = ( *num - 1 ) % option->num_options;
            }
            func = menu->options[menu->current_selection].callback_func;
            if ( func && func(menu,menu->app,-1) == 0 ) {
                delete_menu(menu);
                parent->child = NULL;	
            }      
	    menu_display(parent);

        }
        break;
    case kBack:
        if ( menu->parent ) {
            menu_t *parent = menu->parent;
            delete_menu(menu);
            parent->child = NULL;
            menu_display(parent);
        }
        break;
    case kGo:
        while ( menu->parent ) {
            menu_t *parent = menu->parent;
            delete_menu(menu);
            menu = parent;
            menu->child = NULL;
        }
        menu_display(menu);
        break;
    case kOk:
        if ( menu->options != NULL ) {
            menu_t  *parent;
            func = menu->options[menu->current_selection].callback_func;
            if ( func && func(menu,menu->app,menu->current_selection) == 0 ) {
                parent = menu->parent;
                delete_menu(menu);
                parent->child = NULL;	
                menu_display(parent);
            }        
        }
        break;
    case k1 ... k9:
        if ( menu->flags & MENU_NUMBERS ) {
            pos = code - k1;
            if ( pos < menu->numoptions && menu->options ) {
                func = menu->options[pos].callback_func;
                if ( func && func(menu,menu->app,pos) == 0 ) {
                    delete_menu(menu);
                }
            }
        }
        break;
    default:
        printf("Received IR code %d - not used here\n",code);
    }
    return 0;
}


 
/*
 * Local Variables:
 *  indent-tabs-mode:nil
 *  require-final-newline:t
 *  c-basic-offset: 4
 * End:
 */
